/*
 *  Brno University of Technology
 *
 *  Copyright (C) 2009-2010 by Brno University of Technology and the contributors
 *
 *  Complete list of developers available at our web site:
 *
 *       http://spl.utko.feec.vutbr.cz
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.
 *
 *  You should have received a copy of the GNU Affero General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/.
 */
package cz.vutbr.feec.imageprocessing.op.featureExtraction;

import java.util.List;

import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.operator.OperatorException;
import com.rapidminer.parameter.ParameterType;
import com.rapidminer.parameter.ParameterTypeInt;

import cz.vutbr.feec.imageprocessing.imagej.imageplus.ImagePlusGSIOObject;

public class AutocovarianceOperator extends BaseImageOperator {

	private static final String PARAM_SHIFT_X = "shift_x";
	private static final String PARAM_SHIFT_Y = "shift_y";
	private static final String PARAM_REAL_WINDOW_SIZE = "window_size";

	public AutocovarianceOperator(OperatorDescription description) {
		super(description);
	}

	private double autoCovCalc(int height, int width, int shiftX, int shiftY) {

		double res = 0;
		try {

			/*
			 * Podle skript MCSI strana 42
			 */
			double mean = 0;
			double meanShifted = 0;
			double pixelCount = 0;
			for (int y = -height / 2; y < height / 2 + height % 2; y++) {
				for (int x = -width / 2; x < width / 2 + width % 2; x++) {
					mean += getCentralPixel(x, y);
					meanShifted += getCentralPixel(x + shiftX, y + shiftY);
					pixelCount++;
				}
			}
			mean = mean / pixelCount;
			meanShifted = meanShifted / pixelCount;
			// probability of symbol
			double p = 1.0 / 256.0;

			for (int y = -height / 2; y < height / 2; y++) {
				for (int x = -width / 2; x < width / 2; x++) {
					res += (getCentralPixel(x, y) - mean)
							* (getCentralPixel(x + shiftX, y + shiftY) - meanShifted)
							* p;
				}
			}

			/*
			 * podle clanku Autocovariance-based Perceptual Textural Features
			 * Corresponding to Human Visual Perception
			 */

			// double pixelCount = 0;
			// for (int y = -height / 2; y < height / 2; y++) {
			// for (int x = width / 2; x < width / 2; x++) {
			// if (isInnerPixel(x + shiftX, y + shiftY)) {
			// res += getCentralPixel(x, y)
			// * getCentralPixel(x + shiftX, y + shiftY);
			// pixelCount++;
			// }
			// }
			// }
			// if(pixelCount != 0){
			// res = res / pixelCount;
			// }

		} catch (OperatorException e) {
			e.printStackTrace();
		}
		return res;
	}

	@Override
	public void doWork() throws OperatorException {
		int shiftX = getParameterAsInt(PARAM_SHIFT_X);
		int shiftY = getParameterAsInt(PARAM_SHIFT_Y);
		int winSize = getParameterAsInt(PARAM_REAL_WINDOW_SIZE);
		ImagePlusGSIOObject imObj = inImg.getData();

		int winHeight = imObj.getWindowHeight();
		int winWidth = imObj.getWindowWidth();
		if(winHeight < winSize || winWidth < winSize){
			throw new OperatorException("Window dimension must be smaller then window in Feature Extraction.");
		}else if (winSize > 0) {
			winHeight = winSize;
			winWidth = winSize;
		} else if (winSize < 0) {
			throw new OperatorException("Bad window dimension.");
		}

		double result = autoCovCalc(winHeight, winWidth, shiftX, shiftY);

		setFeature("AutoCoVariance", result);

		outExSet.deliver(features);
	}

	@Override
	public List<ParameterType> getParameterTypes() {
		List<ParameterType> types = super.getParameterTypes();
		types.add(new ParameterTypeInt(PARAM_SHIFT_X, "Shift in x direction",
				0, 999999999, 0));
		types.add(new ParameterTypeInt(PARAM_SHIFT_Y, "Shift in Y direction",
				0, 999999999, 0));
		types.add(new ParameterTypeInt(PARAM_REAL_WINDOW_SIZE, "window size",
				2, 99999, 2));
		return types;
	}
}
