/*
 *  Brno University of Technology
 *
 *  Copyright (C) 2009-2010 by Brno University of Technology and the contributors
 *
 *  Complete list of developers available at our web site:
 *
 *       http://spl.utko.feec.vutbr.cz
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.
 *
 *  You should have received a copy of the GNU Affero General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/.
 */
package cz.vutbr.feec.imageprocessing.op.featureExtraction;

import ij.ImagePlus;

import java.util.List;

import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.operator.OperatorException;
import com.rapidminer.operator.UserError;
import com.rapidminer.parameter.ParameterType;
import com.rapidminer.parameter.ParameterTypeInt;
import com.rapidminer.parameter.UndefinedParameterError;

import cz.vutbr.feec.imageprocessing.imagej.imageplus.ImagePlusGSIOObject;

// TODO: Auto-generated Javadoc
/**
 * The Class DiagonalHaarLikeFeature.
 */
public class DiagonalHaarLikeFeature extends BaseImageOperator
{

	/** The PARA m_ windo w_ size. */
	private final String PARAM_WINDOW_SIZE = "Window size";
	
	/** The PARA m_ x_ position. */
	private final String PARAM_X_POSITION = "X position";
	
	/** The PARA m_ y_ position. */
	private final String PARAM_Y_POSITION = "Y position";

	/**
	 * Instantiates a new diagonal haar like feature.
	 * 
	 * @param description the description
	 */
	public DiagonalHaarLikeFeature(OperatorDescription description)
	{
		super(description);
		// TODO Auto-generated constructor stub
	}
	
	

	/* (non-Javadoc)
	 * @see com.rapidminer.operator.Operator#doWork()
	 */
	@Override
	public void doWork() throws OperatorException
	{
		clearFeatures();  
		ImagePlusGSIOObject img = inImg.getData();
		ImagePlus im = img.getImage();		
		
		int xPos = img.getWindowXPosition();
		int yPos = img.getWindowYPosition();
		
		int winSize;		
		try	{
			winSize = getParameterAsInt(PARAM_WINDOW_SIZE);	
		}
		catch (UndefinedParameterError e) {
			winSize = img.getWindowWidth();
		}		
		
		//check the parameter values
		if(winSize%2 != 0)
			throw new UserError(this, 116, "The window size has to be an even value");
		if(im.getProcessor().getPixel(0, 0)  < 0) 
			throw new UserError(this, 116, "Image is not grayscale.");
		
		int black = 0;
		int white = 0;
		//top left corner
		for (int y = yPos; y < yPos + winSize/2; y++)
		{
			for (int x = xPos; x < xPos + winSize/2; x++)
			{
				white += im.getProcessor().getPixel(x, y);
			}
		}
		//top right corner
		for (int y = yPos; y < yPos + winSize/2; y++)
		{
			for (int x = xPos + winSize/2; x < xPos + winSize; x++)
			{
				black += im.getProcessor().getPixel(x, y);
			}
		}
		//bottom left corner
		for (int y = yPos + winSize/2; y < yPos + winSize; y++)
		{
			for (int x = xPos; x < xPos + winSize/2; x++)
			{
				black += im.getProcessor().getPixel(x, y);
			}
		}
		//bottom right corner
		for (int y = yPos + winSize/2; y < yPos + winSize; y++)
		{
			for (int x = xPos + winSize/2; x < xPos + winSize; x++)
			{
				white += im.getProcessor().getPixel(x, y);
			}
		}
		int sum = black - white;
		setFeature("DiagonalHF", sum);
		
		outExSet.deliver(features);
	}
	
	/* (non-Javadoc)
	 * @see cz.vutbr.feec.imageprocessing.op.featureExtraction.BaseImageOperator#getParameterTypes()
	 */
	@Override
	public List<ParameterType> getParameterTypes()
	{
		
		List<ParameterType> params =  super.getParameterTypes();
		params.add(new ParameterTypeInt(
				PARAM_WINDOW_SIZE, 
				"Size of the Haar like feature. It has to be an even value.", 
				2, Integer.MAX_VALUE, 8, true));
		
		return params;
	}

}
