/*
 *  Brno University of Technology
 *
 *  Copyright (C) 2009-2010 by Brno University of Technology and the contributors
 *
 *  Complete list of developers available at our web site:
 *
 *       http://spl.utko.feec.vutbr.cz
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.
 *
 *  You should have received a copy of the GNU Affero General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/.
 */
package cz.vutbr.feec.imageprocessing.op.featureExtraction;

import ij.ImagePlus;

import java.util.List;

import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.operator.OperatorException;
import com.rapidminer.operator.UserError;
import com.rapidminer.parameter.ParameterType;
import com.rapidminer.parameter.ParameterTypeCategory;
import com.rapidminer.parameter.ParameterTypeInt;

import cz.vutbr.feec.imageprocessing.imagej.imageplus.ImagePlusGSIOObject;

// TODO: Auto-generated Javadoc
/**
 * The Class EdgeHaarLikeFeature.
 */
@Deprecated // Radim - mam podezreni, ze je to spatne implementovano - 
// neni to zrejme v soouladu s tim, jak funguje Feature Extraction operator
public class EdgeHaarLikeFeature extends BaseImageOperator {

	/**
	 * Instantiates a new edge haar like feature.
	 * 
	 * @param description the description
	 */
	public EdgeHaarLikeFeature(OperatorDescription description) {
		super(description);
		// TODO Auto-generated constructor stub
	}

	/** The Constant PARAM_TYPE_OF_FEATURE. */
	private static final String PARAM_TYPE_OF_FEATURE = "Type of feature";
	
	/** The Constant PARAM_FEATURES. */
	private static final String[] PARAM_FEATURES = new String[] {
			"Vertical (||)", "Horizontal (=)" };
	
	/** The Constant PARAM_POSITION_X. */
	private static final String PARAM_POSITION_X = "X position";
	
	/** The Constant PARAM_POSITION_Y. */
	private static final String PARAM_POSITION_Y = "Y position";
	
	/** The Constant PARAM_SIZE_X. */
	private static final String PARAM_SIZE_X = "X size";
	
	/** The Constant PARAM_SIZE_Y. */
	private static final String PARAM_SIZE_Y = "Y size";

	/* (non-Javadoc)
	 * @see com.rapidminer.operator.Operator#doWork()
	 */
	@Override
	public void doWork() throws OperatorException {
		// TODO Auto-generated method stub
		// super.doWork();
		clearFeatures();
		// loading variables
		int positionX = getParameterAsInt(PARAM_POSITION_X);
		int positionY = getParameterAsInt(PARAM_POSITION_Y);
		int sizeX = getParameterAsInt(PARAM_SIZE_X);
		int sizeY = getParameterAsInt(PARAM_SIZE_Y);
		boolean haar = false;
		if (getParameterAsInt(PARAM_TYPE_OF_FEATURE) == 0)
			haar = true;
		// loading window and picture
		ImagePlusGSIOObject img = inImg.getData();
		int winHeight = img.getWindowHeight();
		int winWidth = img.getWindowWidth();
		int winX = img.getWindowXPosition();
		int winY = img.getWindowYPosition();
		ImagePlus image = img.getImage();
		// declaration variables for output
		int white = 0;
		int black = 0;
		int output = 0;
		boolean temp = false;

		// if the haar's area is in the window
		if (!((winWidth >= (positionX + sizeX)) && (winHeight >= (positionY + sizeY))))
			throw new UserError(this, 116, "Feature is out of window.");
		// if size of area is valid
		if (!(sizeX % 2 == 0 && sizeX > 0 && sizeY > 0))
			throw new UserError(this, 116,
					"Value of \"X size\" or \"Y size\" is invalid.");

		//algorithm of Edge Haar-like feature  
		for (int i = (winY + positionY); i < ((winY + positionY) + sizeY); i++) {
			for (int j = (winX + positionX); j < ((winX + positionX) + sizeX); j++) {
				if (haar)
					temp = (j < ((winX + positionX) + sizeX / 2));
				else
					temp = (i < ((winY + positionY) + sizeY / 2));

				if (temp)
					white += image.getProcessor().get(j, i);
				else
					black += image.getProcessor().get(j, i);
			}
		}

		output = white - black;
		setFeature("EdgeHF", (double) (output));
		outExSet.deliver(features);

	}

	/* (non-Javadoc)
	 * @see cz.vutbr.feec.imageprocessing.op.featureExtraction.BaseImageOperator#getParameterTypes()
	 */
	@Override
	public List<ParameterType> getParameterTypes() {
		List<ParameterType> types = super.getParameterTypes();
		types.add(new ParameterTypeCategory(PARAM_TYPE_OF_FEATURE,
				"Type of feature", PARAM_FEATURES, 0));
		types.add(new ParameterTypeInt(PARAM_POSITION_X, "X position", 0,
				Integer.MAX_VALUE, 0));
		types.add(new ParameterTypeInt(PARAM_POSITION_Y, "Y position", 0,
				Integer.MAX_VALUE, 0));
		types.add(new ParameterTypeInt(PARAM_SIZE_X, "X size", 1,
				Integer.MAX_VALUE, 2));
		types.add(new ParameterTypeInt(PARAM_SIZE_Y, "Y size", 1,
				Integer.MAX_VALUE, 1));

		return types;

	}

}
