/*
 *  Brno University of Technology
 *
 *  Copyright (C) 2009-2010 by Brno University of Technology and the contributors
 *
 *  Complete list of developers available at our web site:
 *
 *       http://spl.utko.feec.vutbr.cz
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.
 *
 *  You should have received a copy of the GNU Affero General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/.
 */
package cz.vutbr.feec.imageprocessing.op.featureExtraction;

import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import com.rapidminer.example.Attribute;
import com.rapidminer.example.Attributes;
import com.rapidminer.example.ExampleSet;
import com.rapidminer.example.table.AttributeFactory;
import com.rapidminer.example.table.DataRow;
import com.rapidminer.example.table.DataRowFactory;
import com.rapidminer.example.table.MemoryExampleTable;
import com.rapidminer.operator.IOObject;
import com.rapidminer.operator.OperatorChain;
import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.operator.OperatorException;
import com.rapidminer.operator.UserError;
import com.rapidminer.operator.ports.InputPort;
import com.rapidminer.operator.ports.InputPortExtender;
import com.rapidminer.operator.ports.OutputPort;
import com.rapidminer.operator.ports.OutputPortExtender;
import com.rapidminer.operator.ports.metadata.AttributeMetaData;
import com.rapidminer.operator.ports.metadata.ExampleSetMetaData;
import com.rapidminer.operator.ports.metadata.GenerateNewMDRule;
import com.rapidminer.operator.ports.metadata.MetaData;
import com.rapidminer.operator.ports.metadata.OneToManyPassThroughRule;
import com.rapidminer.operator.ports.metadata.SimplePrecondition;
import com.rapidminer.operator.ports.metadata.SubprocessTransformRule;
import com.rapidminer.parameter.ParameterType;
import com.rapidminer.parameter.ParameterTypeBoolean;
import com.rapidminer.parameter.ParameterTypeCategory;
import com.rapidminer.parameter.ParameterTypeDouble;
import com.rapidminer.parameter.ParameterTypeInt;
import com.rapidminer.parameter.conditions.EqualTypeCondition;
import com.rapidminer.tools.Ontology;
import com.rapidminer.tools.RandomGenerator;

import cz.vutbr.feec.imageprocessing.imagej.imageplus.ImagePlusGSIOObject;

/**
 * This operator is using sliding window over an image and extracts features as
 * an example set. According to the label image evaluates the samples and
 * assigns the label for the features.
 * 
 */
public class FeatureExtractionOperator extends OperatorChain {
	// vnejsi vstup (velky obrazek)
	private InputPort imgIn = getInputPorts()
			.createPort("grayscale image plus");
	// vnitrni zdroj (okno z puvodniho obrazku)
	private OutputPortExtender innerOut = new OutputPortExtender("grayscale",
			getSubprocess(0).getInnerSources());
	// vnitrni vystup (okno z puvodniho obrazku)
	protected InputPortExtender innerIn = new InputPortExtender("feature",
			getSubprocess(0).getInnerSinks(), new MetaData(
					FeatureIOObject.class), true);
	// celkovy vystup
	private OutputPort exsOut = getOutputPorts().createPort("example set");

	private static final String PARAM_WINDOW_WIDTH = "window_width";
	private static final String PARAM_WINDOW_HEIGHT = "window_height";
	private static final String PARAM_WINDOW_STEP = "window_step";
	private static final String PARAM_PROB_WHITE = "prob_white";
	private static final String PARAM_PROB_BLACK = "prob_black";
	private static final String PARAM_INCLUDE_POSITION = "include_position_in_img";
	private static final String PARAM_INCLUDE_FILENAME = "include_position_filename";
	private static final String PARAM_SELECTION = "selection_type";
	private static final String[] TYPES = { "BY SIZE OF IMAGE",
			"GIVEN NUMBER OF SAMPLES" };
	private static final String PARAM_NUMBER_OF_SAMPLES = "number_of_samples";

	List<Attribute> specialAttributes = new Vector<Attribute>();

	private ExampleSet exampleSet = null;

	public FeatureExtractionOperator(OperatorDescription description)
			throws OperatorException {
		super(description, "Executed process");
		innerIn.start();
		innerOut.start();

		imgIn.addPrecondition(new SimplePrecondition(imgIn, new MetaData(
				ImagePlusGSIOObject.class)));

		getTransformer()
				.addRule(
						new OneToManyPassThroughRule(imgIn, innerOut
								.getManagedPorts()));
		getTransformer().addGenerationRule(exsOut, ExampleSet.class);
		getTransformer().addRule(new SubprocessTransformRule(getSubprocess(0)));
		// innerIn.makePassThroughRule(exsOut);

		// add metadata
		getTransformer().addRule(
				new GenerateNewMDRule(exsOut, new ExampleSetMetaData()) {
					@Override
					public MetaData modifyMetaData(MetaData unmodifiedMetaData) {
						// construction of meta data
						ExampleSetMetaData emd = (ExampleSetMetaData) unmodifiedMetaData;
						emd = addMetaDataAttributes(emd);

						return emd;
					}
				});
	}

	protected ExampleSetMetaData addMetaDataAttributes(ExampleSetMetaData emd) {
		try {
			AttributeMetaData amd = new AttributeMetaData(
					Attributes.LABEL_NAME, getProvidedLabelType(),
					Attributes.LABEL_NAME);
			emd.addAttribute(amd);
		} catch (UserError e) {
		}
		return emd;
	}

	private void convertFeaturesToExampleSet(ExampleSet features2,
			InputPortExtender innerIn, boolean includeFileName,
			boolean includePositionInImage, ImagePlusGSIOObject imageIO) {
		// Pokud je volano poprve, musi se nejprve vytvorit sloupce (atributy)
		char decimalSeperator = '.';
		DataRowFactory ROW_FACTORY = new DataRowFactory(0, decimalSeperator);
		MemoryExampleTable table = (MemoryExampleTable) exampleSet
				.getExampleTable();

		// Number of attributes
		int numOfAttrib = table.getNumberOfAttributes();

		// create data vector
		Object[] values = new Object[numOfAttrib];

		// nastav hodnoty z kazdeho sink vystupu
		int i = 0;
		for (IOObject o : innerIn.getData(false)) {
			FeatureIOObject ef = (FeatureIOObject) o;
			for (Feature feature : ef) {
				values[i] = feature.getValue();
				i++;
			}
		}
		if (includeFileName) {
			values[i] = imageIO.getFileName();
			i++;
		}
		if (includePositionInImage) {
			values[i] = imageIO.getWindowXPosition();
			i++;
			values[i] = imageIO.getWindowYPosition();
			i++;
		}
		if (imageIO.getFeatures() != null) {
			Iterator<Feature> it = imageIO.getFeatures().iterator();
			while (it.hasNext()) {
				Feature feature = it.next();
				values[i] = feature.getValue();
				i++;
			}
		}

		// vloz zazn am do dat
		DataRow row = ROW_FACTORY.create(values, table.getAttributes());
		table.addDataRow(row);
	}

	@Override
	public void doWork() throws OperatorException {
		int winWidth = getParameterAsInt(PARAM_WINDOW_WIDTH);
		int winHeight = getParameterAsInt(PARAM_WINDOW_HEIGHT);
		int winStep = getParameterAsInt(PARAM_WINDOW_STEP);

		double probWhite = getParameterAsDouble(PARAM_PROB_WHITE);
		double probBlack = getParameterAsDouble(PARAM_PROB_BLACK);

		boolean includeFileName = getParameterAsBoolean(PARAM_INCLUDE_FILENAME);
		boolean includePositionInImage = getParameterAsBoolean(PARAM_INCLUDE_POSITION);

		int seed = getParameterAsInt(RandomGenerator.PARAMETER_LOCAL_RANDOM_SEED);

		RandomGenerator rg;
		if (seed == -1) {
			// seed =
			// getParameterAsInt(RandomGenerator.PARAMETER_LOCAL_RANDOM_SEED);
			rg = RandomGenerator.getRandomGenerator(false, seed);
		} else {
			rg = RandomGenerator.getRandomGenerator(true, seed);
		}

		ImagePlusGSIOObject imageIO = imgIn.getData();
		int imgWidth = imageIO.getImage().getWidth();
		int imgHeight = imageIO.getImage().getHeight();

		// to obtain headers - one execution is necessary and these values have
		// to be defined
		imageIO.setWindowWidth(winWidth);
		imageIO.setWindowHeight(winHeight);

		final boolean NO_CLONE = false;
		if (getParameterAsInt(PARAM_SELECTION) == 1) {
			// select given number of samples
			exampleSet = perapreExampleSetHeader(innerIn, includeFileName,
					includePositionInImage, imageIO.getFeatures());
			for (int i = 1; i < getParameterAsInt(PARAM_NUMBER_OF_SAMPLES); i++) {
				// prevzit vystupni data a vlozit je do vystupu
				convertFeaturesToExampleSet(exampleSet, innerIn,
						includeFileName, includePositionInImage, imageIO);
			}
		} else {
			// select random number of samples according to size of the image
			// obtain values
			for (int x = winWidth / 2; x <= imgWidth - winWidth / 2 - winWidth
					% 2; x += winStep) {
				for (int y = winHeight / 2; y <= imgHeight - winHeight / 2
						- winHeight % 2; y += winStep) {
					checkForStop();

					// nastav okno v obrazku
					imageIO.setWindowXPosition(x);
					imageIO.setWindowYPosition(y);

					// nema se preskocit ?
					double fitness = imageIO.getFitnessValue(x, y);
					if (fitness < 128) { // black
						if (probBlack < rg.nextDouble()) {
							// preskoc
							continue;
						}
					} else { // white
						if (probWhite < rg.nextDouble()) {
							// preskoc
							continue;
						}
					}
					// na vsechny vnitrni vstupy vlozim vstupni okno
					innerOut.deliverToAll(imageIO, NO_CLONE);

					// spusti vnitrni operatory
					getSubprocess(0).execute();

					// oznami GUI ze jsme v podprocesu
					inApplyLoop();

					// obtain attributes
					if (exampleSet == null) {
						exampleSet = perapreExampleSetHeader(innerIn,
								includeFileName, includePositionInImage,
								imageIO.getFeatures());
					} else {
						// prevzit vystupni data a vlozit je do vystupu
						convertFeaturesToExampleSet(exampleSet, innerIn,
								includeFileName, includePositionInImage,
								imageIO);
					}
				}
				// Vlozit prevzata data do vysledne example set
				exsOut.deliver(exampleSet);
			}
		}
		// Tady se predava vysledna example set na celkovy vystup
		exampleSet = null;

		// nakonec zrus okno
		imageIO.setWindowXPosition(-1);
		imageIO.setWindowYPosition(-1);
		imageIO.setWindowWidth(-1);
		imageIO.setWindowHeight(-1);
	}

	@Override
	public List<ParameterType> getParameterTypes() {
		List<ParameterType> types = super.getParameterTypes();
		types.add(new ParameterTypeInt(PARAM_WINDOW_WIDTH,
				"Width of the sliding window over the image", 1,
				Integer.MAX_VALUE, 9));
		types.add(new ParameterTypeInt(PARAM_WINDOW_HEIGHT,
				"Height of the sliding window over the image", 1,
				Integer.MAX_VALUE, 9));
		types.add(new ParameterTypeCategory(PARAM_SELECTION, "Selection type",
				TYPES, 0));
		ParameterType p = new ParameterTypeInt(PARAM_WINDOW_STEP,
				"Step size of the sliding window", 1, Integer.MAX_VALUE, 1);
		p.registerDependencyCondition(new EqualTypeCondition(this,
				PARAM_SELECTION, TYPES, true, 0));
		types.add(p);
		p = new ParameterTypeDouble(PARAM_PROB_WHITE,
				"Probability for selecting white", 0.0, 1.0, 1.0);
		p.registerDependencyCondition(new EqualTypeCondition(this,
				PARAM_SELECTION, TYPES, true, 0));
		types.add(p);
		p = new ParameterTypeDouble(PARAM_PROB_BLACK,
				"Probability for selecting black", 0.0, 1.0, 1.0);
		p.registerDependencyCondition(new EqualTypeCondition(this,
				PARAM_SELECTION, TYPES, true, 0));
		types.add(p);
		p = new ParameterTypeInt(PARAM_NUMBER_OF_SAMPLES,
				"Number of samples to take", 0, Integer.MAX_VALUE, 100);
		p.registerDependencyCondition(new EqualTypeCondition(this,
				PARAM_SELECTION, TYPES, true, 1));
		types.add(p);
		types.add(new ParameterTypeInt(
				RandomGenerator.PARAMETER_LOCAL_RANDOM_SEED,
				"Random number generator seed. Chose -1 for global config.",
				-1, 9999999, -1));
		types.add(new ParameterTypeBoolean(PARAM_INCLUDE_FILENAME,
				"Include ignored atribute with image file name.", true));
		types.add(new ParameterTypeBoolean(PARAM_INCLUDE_POSITION,
				"Include position in image where the features were obtined.",
				true));
		types.get(0).setExpert(false);
		types.get(1).setExpert(false);
		types.get(2).setExpert(false);
		return types;
	}

	protected int getProvidedLabelType() throws UserError {
		return Ontology.REAL;
	}

	/**
	 * Pripravi strukturu Example set - vytvori patricne atributy
	 * 
	 * @param includePositionInImage
	 * @param includeFileName
	 * @param featureIOObject
	 * @param imageIO
	 * 
	 * @param innerIn2
	 * 
	 * @return the example set
	 */
	private ExampleSet perapreExampleSetHeader(InputPortExtender innerIn,
			boolean includeFileName, boolean includePositionInImage,
			FeatureIOObject featureIOObject) {
		List<Attribute> attributes = new Vector<Attribute>();
		// Pro kazdy innerIn (vnitrni sink)
		Vector<Attribute> specialAttribs = new Vector<Attribute>();
		Attribute label = null;
		for (IOObject o : innerIn.getData(false)) {
			FeatureIOObject ef = (FeatureIOObject) o;
			// pro kazdy obdrzeny parametr
			for (Feature feature : ef) {
				Attribute att = AttributeFactory.createAttribute(feature
						.getName(), feature.getOntology());
				attributes.add(att);
				// marked attributes will not be used for training
				if (feature.isSpecial()) {
					specialAttribs.add(att);
				}
				if (feature.isLabel()) {
					att.setName(Attributes.LABEL_NAME);
					label = att;
				}
			}
		}

		// include file name ?
		if (includeFileName) {
			Attribute att = AttributeFactory.createAttribute("FileName",
					Ontology.STRING);
			attributes.add(att);
			specialAttribs.add(att);
		}

		// include image position ?
		if (includePositionInImage) {
			Attribute attX = AttributeFactory.createAttribute("PositionX",
					Ontology.INTEGER);
			Attribute attY = AttributeFactory.createAttribute("PositionY",
					Ontology.INTEGER);
			attributes.add(attX);
			attributes.add(attY);
			specialAttribs.add(attX);
			specialAttribs.add(attY);
		}

		// are there some features from external operators?
		if (featureIOObject != null) {
			Iterator<Feature> it = featureIOObject.iterator();
			while (it.hasNext()) {
				Feature feature = it.next();
				Attribute att = AttributeFactory.createAttribute(feature
						.getName(), feature.getOntology());
				attributes.add(att);
				// marked attributes will not be used for training
				if (feature.isSpecial()) {
					specialAttribs.add(att);
				}
				if (feature.isLabel()) {
					att.setName(Attributes.LABEL_NAME);
					label = att;
				}
			}
		}

		MemoryExampleTable table = new MemoryExampleTable(attributes);

		ExampleSet es = table.createExampleSet();

		// Mark fitness as label
		if (label != null) {
			// always the same name
			// label.setName(Attributes.LABEL_NAME);
			es.getAttributes().setLabel(label);
		}

		// mark attributes to be ignored
		for (Attribute attribute : specialAttribs) {
			es.getAttributes().setSpecialAttribute(attribute,
					attribute.getName());
		}
		return es;
	}
}
