/*
 *  Brno University of Technology
 *
 *  Copyright (C) 2009-2010 by Brno University of Technology and the contributors
 *
 *  Complete list of developers available at our web site:
 *
 *       http://spl.utko.feec.vutbr.cz
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.
 *
 *  You should have received a copy of the GNU Affero General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/.
 */
package cz.vutbr.feec.imageprocessing.op.featureExtraction;

import java.util.Iterator;
import java.util.TreeMap;

import com.rapidminer.operator.ResultObjectAdapter;
import com.rapidminer.tools.Ontology;

/**
 * The Class FeatureIOObject.
 */
public class FeatureIOObject extends ResultObjectAdapter implements
		Iterable<Feature> {

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = -77981442928741385L;
	
	/** The features. */
	private TreeMap<String, Feature> features = new TreeMap<String, Feature>();

	/**
	 * Instantiates a new feature io object.
	 */
	public FeatureIOObject() {
	}
	
	public TreeMap<String, Feature> getFeatures(){
		return this.features;
	}

	/**
	 * Clear.
	 */
	public void clear() {
		features.clear();
	}

	/* (non-Javadoc)
	 * @see java.lang.Iterable#iterator()
	 */
	@Override
	public Iterator<Feature> iterator() {
		return features.values().iterator();
	}

	/**
	 * Sets the feature.
	 * 
	 * @param name the name
	 * @param value the value
	 */
	public void setFeature(String name, double value) {
		features.put(name, new Feature(name, value, Ontology.REAL,
				FeatureType.ATTRIBUTE));
	}

	/**
	 * Sets the feature.
	 * 
	 * @param name the name
	 * @param value the value
	 * @param ontology the ontology
	 * @param type the type
	 */
	public void setFeature(String name, Object value, int ontology,
			FeatureType type) {
		features.put(name, new Feature(name, value, ontology, type));
	}

	/* (non-Javadoc)
	 * @see com.rapidminer.operator.ResultObjectAdapter#toResultString()
	 */
	@Override
	public String toResultString() {
		String s = "";
		for (Feature r : features.values()) {
			s += r.getName() + ":" + r.getValue() + ",\n";
		}
		return "FeatureIOObject: " + s;
	}

	/**
	 * X coordinate.
	 * 
	 * @param name the name
	 * @param value the value
	 * @param ontology the ontology
	 * @param type the type
	 */
	public void x(String name, Object value, int ontology, FeatureType type) {
		features.put(name, new Feature(name, value, ontology, type));
	}

	public void addFeatures(FeatureIOObject features2) {
			this.features.putAll(features2.getFeatures());
	}

}
