/*
 *  Brno University of Technology
 *
 *  Copyright (C) 2009-2010 by Brno University of Technology and the contributors
 *
 *  Complete list of developers available at our web site:
 *
 *       http://spl.utko.feec.vutbr.cz
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.
 *
 *  You should have received a copy of the GNU Affero General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/.
 */
package cz.vutbr.feec.imageprocessing.op.featureExtraction;

import java.util.List;

import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.operator.OperatorException;
import com.rapidminer.parameter.ParameterType;
import com.rapidminer.parameter.ParameterTypeBoolean;
import com.rapidminer.tools.Ontology;

import cz.vutbr.feec.imageprocessing.imagej.imageplus.ImagePlusGSIOObject;

/**
 * Tento operator vraci hodnotu fitness na zaklade parametru FitnessMask, ktery
 * je nastaven pri nacitani obrazku.
 */
public class FitnessValueExtractor extends BaseImageOperator {

	private static final String PARAM_IS_LABEL = "is_label";

	public FitnessValueExtractor(OperatorDescription description) {
		super(description);
	}

	@Override
	public void doWork() throws OperatorException {
		// delete all previous records
		clearFeatures();

		ImagePlusGSIOObject ioIm = inImg.getData();

		int x = ioIm.getWindowXPosition();
		int y = ioIm.getWindowYPosition();
		double res = ioIm.getFitnessValue(x, y);

		boolean isLabel = getParameterAsBoolean(PARAM_IS_LABEL);

		// add feature
		if (isLabel) {
			setFeature("Fitness", res, Ontology.REAL, FeatureType.LABEL);
		} else {
			setFeature("Fitness", res);
		}

		// deliver output
		outExSet.deliver(features);
	}

	@Override
	public List<ParameterType> getParameterTypes() {
		List<ParameterType> types = super.getParameterTypes();
		types.add(new ParameterTypeBoolean(PARAM_IS_LABEL,
				"Should the attribute be marked as a label?", true));
		return types;
	}
}
