/*
 *  Brno University of Technology
 *
 *  Copyright (C) 2009-2010 by Brno University of Technology and the contributors
 *
 *  Complete list of developers available at our web site:
 *
 *       http://spl.utko.feec.vutbr.cz
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.
 *
 *  You should have received a copy of the GNU Affero General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/.
 */
package cz.vutbr.feec.imageprocessing.op.featureExtraction;

import ij.process.ImageProcessor;

import java.util.List;

import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.operator.OperatorException;
import com.rapidminer.parameter.ParameterType;
import com.rapidminer.parameter.ParameterTypeInt;

import cz.vutbr.feec.imageprocessing.imagej.imageplus.ImagePlusGSIOObject;

/**
 * The Class Block Difference of Inverse probabilities.
 */
public class HomogeneityOfLesion extends BaseImageOperator {

	private static final String PARAM_WINDOW_SIZE = "window_size";

	public HomogeneityOfLesion(OperatorDescription description) {
		super(description);
	}

	@Override
	public void doWork() throws OperatorException {
		ImagePlusGSIOObject ioimage = inImg.getData();
		ImageProcessor window = ioimage.getImage().getProcessor();
		// window parameters
		int width = ioimage.getWindowWidth();
		int height = ioimage.getWindowHeight();
		// recalculate window parameters
		int winSize = getParameterAsInt(PARAM_WINDOW_SIZE);
		if(height < winSize || width < winSize){
			throw new OperatorException("Window dimension must be smaller then window in Feature Extraction.");
		}else if (winSize > 0) {
			height = winSize;
			width = winSize;
		} else if (winSize < 0) {
			throw new OperatorException("Bad window dimension.");
		}
		int x = ioimage.getWindowXPosition() - width/2 - width%2 + 1;
		int y = ioimage.getWindowYPosition() - height/2 - height%2+ 1;
		// set window
		window.setRoi(x, y, width, height);
		// get histogram
		int[] histogram = window.getHistogram();
		// total number of pixels
		int pixelCount = height * width;
		// maximal entropy
		double maxEntropy = 8; // log(256, 2); //256 gray levels
		double entropy = 0;
		// entropy calculation
		for (int count : histogram) {
			double p = (double) count / (double) pixelCount;
			if (p != 0) {
				entropy += -p * log(p, 2);
			}
		}
		double relativeEntropy = entropy / maxEntropy;

		// add feature
		setFeature("Homgenity", relativeEntropy);

		// deliver output
		outExSet.deliver(features);
	}

	@Override
	public List<ParameterType> getParameterTypes() {
		List<ParameterType> types = super.getParameterTypes();
		types.add(new ParameterTypeInt(PARAM_WINDOW_SIZE, "Window width", 0,
				999999999, 3));

		return types;
	}

	private double log(double value, double base) {
		return Math.log(value) / Math.log(base);
	}
}
