/*
 *  Brno University of Technology
 *
 *  Copyright (C) 2009-2010 by Brno University of Technology and the contributors
 *
 *  Complete list of developers available at our web site:
 *
 *       http://spl.utko.feec.vutbr.cz
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.
 *
 *  You should have received a copy of the GNU Affero General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/.
 */
package cz.vutbr.feec.imageprocessing.op.featureExtraction;

import ij.ImagePlus;

public class ImageAutoCovariance {

	/**
	 * Covariance.
	 * 
	 * \f[ f(\delta_i,\delta_j) = \frac{1}{(n-\delta_i)(m-\delta_j)}
	 * \sum_{i=0}^{n-\delta_i-1} \sum_{j=0}^{n-\delta_j-1}
	 * I(i,j)I(i+\delta_i,j+\delta_i) \f]
	 * 
	 * 
	 * where \f$\delta_i\f$ is shift in \f$x\f$ axis of the image,
	 * \f$\delta_j\f$ is the shift in y axis of the image, \f$I\f$ is an image
	 * of size \f$m \times n\f$ , I(x,y) is an function returning pixel
	 * intensity at \f$x\f$ and \f$y\f$ position. The range \f$0 <= \delta_i <=
	 * n - 1\f$ and \f$0 <= \delta_j <= m - 1\f$
	 * 
	 * @param img
	 *            the img
	 * @param shiftX
	 *            the shift x
	 * @param shiftY
	 *            the shift y
	 * 
	 * @return the double
	 */
	public static double covariance(ImagePlus img, int shiftX, int shiftY) {
		int width = img.getWidth();
		int height = img.getHeight();

		byte[] data = (byte[]) img.getProcessor().getPixels();

		double res = 0;
		for (int x = 0; x < width; x++) {
			for (int y = 0; y < height; y++) {
				res += data[x + y * width]
						+ data[(x + shiftX) + (y + shiftY) * width];
			}
		}
		res = res / ((width - shiftX) * (height - shiftX));
		return 0;
	}
}
