/*
 *  Brno University of Technology
 *
 *  Copyright (C) 2009-2010 by Brno University of Technology and the contributors
 *
 *  Complete list of developers available at our web site:
 *
 *       http://spl.utko.feec.vutbr.cz
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.
 *
 *  You should have received a copy of the GNU Affero General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/.
 */
package cz.vutbr.feec.imageprocessing.op.featureExtraction;

import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.List;

import au.com.bytecode.opencsv.CSVReader;

import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.operator.OperatorException;
import com.rapidminer.parameter.ParameterType;
import com.rapidminer.parameter.ParameterTypeFile;
import com.rapidminer.tools.Ontology;

import cz.vutbr.feec.imageprocessing.imagej.imageplus.ImagePlusGSIOObject;

/**
 * 1st column: MIAS database reference number. isn't written in exampleSet 2nd
 * column: Character of background tissue: F Fatty G Fatty-glandular D
 * Dense-glandular 3rd column: Class of abnormality present: CALC Calcification
 * CIRC Well-defined/circumscribed masses SPIC Spiculated masses MISC Other,
 * ill-defined masses ARCH Architectural distortion ASYM Asymmetry NORM Normal
 * 4th column: Severity of abnormality; B Benign M Malignant 5th, 6th columns:
 * x,y image-coordinates of centre of abnormality. 7th column: Approximate
 * radius (in pixels) of a circle enclosing the abnormality.
 * 
 */
public class LoadAttributeFromFile extends BaseImageOperator {

	// mdb245 F CALC M *NOTE 3* nahrazeno mdb245 F CALC U 0 0 0
	private static final String PARAMETER_FILENAME = "filename";

	public LoadAttributeFromFile(OperatorDescription description) {
		super(description);
	}

	@Override
	public void doWork() throws OperatorException {
		ImagePlusGSIOObject image = inImg.getData();
		String fileName = removeExtension(image.getFileName());
		try {
			char separator = ' ';
			CSVReader reader = new CSVReader(new FileReader(
					getParameterAsString(PARAMETER_FILENAME)), separator);
			List<String[]> lines = reader.readAll();

			if (lines.size() == 0) {
				throw new OperatorException("No lines found in attribute file.");
			}

			for (String[] line : lines) {
				if (fileName.equalsIgnoreCase(line[0])) {
					setFeature("Tissue", line[1], Ontology.STRING);
					setFeature("Abnormality class", line[2], Ontology.STRING);
					setFeature("Severity", line[3], Ontology.STRING);
					setFeature("X location", Integer.valueOf(line[4]),
							Ontology.INTEGER);
					setFeature("Y location", Integer.valueOf(line[5]),
							Ontology.INTEGER);
					setFeature("radius", Integer.valueOf(line[6]),
							Ontology.INTEGER);

					outExSet.deliver(features);
					// record already found
					return;
				}
			}

		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	@Override
	public List<ParameterType> getParameterTypes() {
		List<ParameterType> types = super.getParameterTypes();

		ParameterType type = new ParameterTypeFile(PARAMETER_FILENAME,
				"File to load attributes from.", "txt", false);
		type.setExpert(false);
		types.add(type);

		return types;
	}

	private String removeExtension(String fileName) {
		return fileName.substring(0, fileName.lastIndexOf("."));
	}
}
