/*
 *  Brno University of Technology
 *
 *  Copyright (C) 2009-2010 by Brno University of Technology and the contributors
 *
 *  Complete list of developers available at our web site:
 *
 *       http://spl.utko.feec.vutbr.cz
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.
 *
 *  You should have received a copy of the GNU Affero General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/.
 */
package cz.vutbr.feec.imageprocessing.op.featureExtraction;

import java.util.List;

import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.operator.OperatorException;
import com.rapidminer.parameter.ParameterType;
import com.rapidminer.parameter.ParameterTypeInt;

import cz.vutbr.feec.imageprocessing.imagej.imageplus.ImagePlusGSIOObject;

/**
 * 
 */
public class MinimumSideDifference extends BaseImageOperator {

	private static final String PARAM_REAL_WINDOW_SIZE = "window_size";

	public MinimumSideDifference(OperatorDescription description) {
		super(description);
	}

	private double average(int height, int width, int shift) {
		double sum = 0;
		double count = 0;

		for (int y = -height / 2; y < height / 2 + height % 2; y++) {
			for (int x = -width / 2; x < width / 2 + width % 2; x++) {
				if (isInnerPixel(x + shift, y)) {
					try {
						sum += getCentralPixel(x + shift, y);
					} catch (OperatorException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					count++;
				} else {
					return 0;
				}
			}
		}
		if (count == 0) {
			return 0;
		}
		return sum / count;
	}

	@Override
	public void doWork() throws OperatorException {
		clearFeatures();
		ImagePlusGSIOObject imObj = inImg.getData();
		int winSize = getParameterAsInt(PARAM_REAL_WINDOW_SIZE);
		int winHeight = imObj.getWindowHeight();
		int winWidth = imObj.getWindowWidth();
		if (winSize > 0) {
			if ((3 * winSize) > winHeight || (3 * winSize) > winWidth) {
				throw new OperatorException("Window dimension must be three times smaller than window in Feature Extraction.");
			} else {
				winWidth = winSize;
				winHeight = winSize;
			}
		} else if (winSize <= 0) {
			throw new OperatorException("Bad window dimension.");
		}
		double centerPos = average(winHeight, winWidth, 0);
		double leftPos = average(winHeight, winWidth, -winWidth);
		double rightPos = average(winHeight, winWidth, winWidth);

		double resultL = centerPos - leftPos;
		double resultR = centerPos - rightPos;

		if (resultL > resultR) {
			setFeature("MinimumSideDifference", resultR);
		} else {
			setFeature("MinimumSideDifference", resultL);

		}

		outExSet.deliver(features);

	}

	@Override
	public List<ParameterType> getParameterTypes() {
		List<ParameterType> types = super.getParameterTypes();
		types.add(new ParameterTypeInt(PARAM_REAL_WINDOW_SIZE, "window size",
				2, 99999, 2));
		return types;
	}
}
