/*
 *  Brno University of Technology
 *
 *  Copyright (C) 2009-2010 by Brno University of Technology and the contributors
 *
 *  Complete list of developers available at our web site:
 *
 *       http://spl.utko.feec.vutbr.cz
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.
 *
 *  You should have received a copy of the GNU Affero General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/.
 */
package cz.vutbr.feec.imageprocessing.op.featureExtraction;

import java.util.List;

import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.operator.OperatorException;
import com.rapidminer.parameter.ParameterType;
import com.rapidminer.parameter.ParameterTypeCategory;

import cz.vutbr.feec.imageprocessing.imagej.imageplus.ImagePlusGSIOObject;

public class RelativePosition extends BaseImageOperator {
	private static final String PARAM_OPERAION = "operation_type";

	public RelativePosition(OperatorDescription description) {
		super(description);
	}

	@Override
	public void doWork() throws OperatorException {
		clearFeatures();
		ImagePlusGSIOObject ioIm = inImg.getData();

		double xrel = ioIm.getWindowXPosition()
				/ ((double) ioIm.getImage().getWidth() - 1);
		double yrel = (ioIm.getWindowYPosition())
				/ ((double) ioIm.getImage().getHeight() - 1);

		// Ktere z os pouzit
		int operation = getParameterAsInt(PARAM_OPERAION);
		if (operation == 0) {
			setFeature("rel_pos_X", xrel);
		} else if (operation == 1) {
			setFeature("rel_pos_Y", yrel);
		} else {
			setFeature("rel_pos_X", xrel);
			setFeature("rel_pos_Y", yrel);
		}

		// deliver output
		outExSet.deliver(features);
	}

	@Override
	public List<ParameterType> getParameterTypes() {
		List<ParameterType> types = super.getParameterTypes();
		String[] list = new String[] { "X axis only", "Y axis only",
				"X and Y axis" };
		types.add(new ParameterTypeCategory(PARAM_OPERAION, "Operation", list,
				2));
		return types;
	}
}
