package cz.vutbr.feec.imageprocessing.op.featureExtraction.global;

import ij.process.ImageProcessor;

import com.rapidminer.operator.Operator;
import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.operator.OperatorException;
import com.rapidminer.operator.ports.InputPort;
import com.rapidminer.operator.ports.OutputPort;
import com.rapidminer.operator.ports.metadata.MetaData;
import com.rapidminer.operator.ports.metadata.SimplePrecondition;
import com.rapidminer.tools.Ontology;

import cz.vutbr.feec.imageprocessing.imagej.imageplus.ImagePlusGSIOObject;
import cz.vutbr.feec.imageprocessing.op.featureExtraction.FeatureIOObject;
import cz.vutbr.feec.imageprocessing.op.featureExtraction.FeatureType;

public class DLogDistanceOperator extends Operator {

	protected InputPort inImg = getInputPorts().createPort(
			"grayscale image plus Hist");

	private OutputPort gsImOutput = getOutputPorts().createPort(
			"grayscale image plus");
	private OutputPort globaFeature = getOutputPorts().createPort(
			"global features");

	public DLogDistanceOperator(OperatorDescription description) {
		super(description);
		inImg.addPrecondition(new SimplePrecondition(inImg, new MetaData(
				ImagePlusGSIOObject.class)));
		getTransformer().addGenerationRule(gsImOutput,
				ImagePlusGSIOObject.class);
		getTransformer().addGenerationRule(globaFeature, FeatureIOObject.class);
	}

	@Override
	public void doWork() throws OperatorException {
		ImagePlusGSIOObject imObj = inImg.getData();
		ImageProcessor ip = imObj.getImage().getProcessor();

		int[] histogram = ip.getHistogram();
		double sum = 0;
		for (int i = 0; i < histogram.length; i = i + 2) {
			sum += Math.abs(histogram[i] - histogram[i + 1]);
		}

		if (sum > 1) {
			sum = Math.log(sum) / Math.log(2) + 1.0;
		} else if (sum > 0 && sum <= 1) {
			sum = 1;
		} else {
			sum = 0;
		}

		imObj.setFeature("dLog", sum, Ontology.REAL, FeatureType.ATTRIBUTE);
		gsImOutput.deliver(imObj);
		FeatureIOObject feature = new FeatureIOObject();
		feature.setFeature("dLog", sum, Ontology.REAL, FeatureType.ATTRIBUTE);
		globaFeature.deliver(feature);
	}
}
