package cz.vutbr.feec.imageprocessing.op.featureExtraction.global;

import java.io.File;
import java.util.List;
import java.util.Vector;

import com.rapidminer.example.Attribute;
import com.rapidminer.example.Attributes;
import com.rapidminer.example.ExampleSet;
import com.rapidminer.example.table.AttributeFactory;
import com.rapidminer.example.table.DataRow;
import com.rapidminer.example.table.DataRowFactory;
import com.rapidminer.example.table.MemoryExampleTable;
import com.rapidminer.operator.IOObject;
import com.rapidminer.operator.Operator;
import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.operator.OperatorException;
import com.rapidminer.operator.UserError;
import com.rapidminer.operator.ports.InputPort;
import com.rapidminer.operator.ports.InputPortExtender;
import com.rapidminer.operator.ports.OutputPort;
import com.rapidminer.operator.ports.metadata.AttributeMetaData;
import com.rapidminer.operator.ports.metadata.ExampleSetMetaData;
import com.rapidminer.operator.ports.metadata.GenerateNewMDRule;
import com.rapidminer.operator.ports.metadata.MetaData;
import com.rapidminer.operator.ports.metadata.SimplePrecondition;
import com.rapidminer.parameter.ParameterType;
import com.rapidminer.parameter.ParameterTypeBoolean;
import com.rapidminer.tools.Ontology;

import cz.vutbr.feec.imageprocessing.imagej.imageplus.ImagePlusGSIOObject;
import cz.vutbr.feec.imageprocessing.op.featureExtraction.Feature;
import cz.vutbr.feec.imageprocessing.op.featureExtraction.FeatureIOObject;

public class FeatureToEsConverter extends Operator {

	List<Attribute> specialAttributes = new Vector<Attribute>();

	private ExampleSet exampleSet = null;

	private static final String PARAM_INCLUDE_FILENAME = "filename";
	private static final String PARAM_INCLUDE_FILEPATH = "file path";
	private static final String PARAM_INCLUDE_FILEPATHNAME = "file path + name";

	protected InputPort imgIn = getInputPorts().createPort(
			"grayscale image plus");
	protected InputPortExtender inFeature = new InputPortExtender(
			"global features", getInputPorts());
	private OutputPort exsOut = getOutputPorts().createPort("example set");

	public FeatureToEsConverter(OperatorDescription description) {
		super(description);

		imgIn.addPrecondition(new SimplePrecondition(imgIn, new MetaData(
				ImagePlusGSIOObject.class)));
		inFeature.start();
		getTransformer().addRule(
				inFeature.makeFlatteningPassThroughRule(exsOut));
		// getTransformer().addRule(inFeature.makePassThroughRule(exsOut));
		getTransformer().addGenerationRule(exsOut, ExampleSet.class);
		getTransformer().addRule(
				new GenerateNewMDRule(exsOut, new ExampleSetMetaData()) {
					@Override
					public MetaData modifyMetaData(MetaData unmodifiedMetaData) {
						// construction of meta data
						ExampleSetMetaData emd = (ExampleSetMetaData) unmodifiedMetaData;
						emd = addMetaDataAttributes(emd);

						return emd;
					}
				});
	}

	protected ExampleSetMetaData addMetaDataAttributes(ExampleSetMetaData emd) {
		try {
			AttributeMetaData amd = new AttributeMetaData(
					Attributes.LABEL_NAME, getProvidedLabelType(),
					Attributes.LABEL_NAME);
			emd.addAttribute(amd);
		} catch (UserError e) {
		}
		return emd;
	}

	protected int getProvidedLabelType() throws UserError {
		return Ontology.REAL;
	}

	@Override
	public void doWork() throws OperatorException {

		exampleSet = perapreExampleSetHeader(inFeature,
				getParameterAsBoolean(PARAM_INCLUDE_FILENAME),
				getParameterAsBoolean(PARAM_INCLUDE_FILEPATH),
				getParameterAsBoolean(PARAM_INCLUDE_FILEPATHNAME));
		// prevzit vystupni data a vlozit je do vystupu
		convertFeaturesToExampleSet(exampleSet, inFeature,
				getParameterAsBoolean(PARAM_INCLUDE_FILENAME),
				getParameterAsBoolean(PARAM_INCLUDE_FILEPATH),
				getParameterAsBoolean(PARAM_INCLUDE_FILEPATHNAME));
		exsOut.deliver(exampleSet);
		// Tady se predava vysledna example set na celkovy vystup
		exampleSet = null;
	}

	@Override
	public List<ParameterType> getParameterTypes() {
		List<ParameterType> types = super.getParameterTypes();
		types.add(new ParameterTypeBoolean(PARAM_INCLUDE_FILENAME,
				"Include file name in output ExampleSet", true));
		types.add(new ParameterTypeBoolean(PARAM_INCLUDE_FILEPATH,
				"Include full file path in output ExampleSet", false));
		types.add(new ParameterTypeBoolean(PARAM_INCLUDE_FILEPATHNAME,
				"Include file path including file name in output ExampleSet",
				false));

		types.get(0).setExpert(false);
		types.get(1).setExpert(true);
		types.get(2).setExpert(true);
		return types;
	}

	private void convertFeaturesToExampleSet(ExampleSet features2,
			InputPortExtender innerIn, boolean includeFileName,
			boolean includePath, boolean includePathFileName)
			throws OperatorException {
		// Pokud je volano poprve, musi se nejprve vytvorit sloupce (atributy)
		char decimalSeperator = '.';
		DataRowFactory ROW_FACTORY = new DataRowFactory(0, decimalSeperator);
		MemoryExampleTable table = (MemoryExampleTable) exampleSet
				.getExampleTable();

		// Number of attributes
		int numOfAttrib = table.getNumberOfAttributes();

		// create data vector
		Object[] values = new Object[numOfAttrib];

		// nastav hodnoty z kazdeho sink vystupu
		int i = 0;
		for (IOObject o : innerIn.getData(false)) {
			FeatureIOObject ef = (FeatureIOObject) o;
			for (Feature feature : ef) {
				values[i] = feature.getValue();
				i++;
			}
		}
		if (includeFileName || includePath || includePathFileName) {
			ImagePlusGSIOObject imageIO = imgIn.getData();
			File file = new File(imageIO.getFileName());
			if (includeFileName) {
				values[i] = file.getName();
				i++;
			}
			if (includePath) {
				if (file.getParent() == null) {
					values[i] = "not set";
				} else {
					values[i] = file.getParent().toString();
				}
				i++;
			}
			if (includePathFileName) {
				values[i] = file.getAbsoluteFile().toString();
				i++;
			}
		}
		// vloz zazn am do dat
		DataRow row = ROW_FACTORY.create(values, table.getAttributes());
		table.addDataRow(row);
	}

	private ExampleSet perapreExampleSetHeader(InputPortExtender innerIn,
			boolean includeFileName, boolean includePath,
			boolean includePathFileName) throws OperatorException {
		List<Attribute> attributes = new Vector<Attribute>();
		// Pro kazdy innerIn (vnitrni sink)
		Vector<Attribute> specialAttribs = new Vector<Attribute>();
		Attribute label = null;
		for (IOObject o : innerIn.getData(false)) {
			FeatureIOObject ef = (FeatureIOObject) o;
			// pro kazdy obdrzeny parametr
			for (Feature feature : ef) {
				Attribute att = AttributeFactory.createAttribute(feature
						.getName(), feature.getOntology());
				attributes.add(att);
				// marked attributes will not be used for training
				if (feature.isSpecial()) {
					specialAttribs.add(att);
				}
				if (feature.isLabel()) {
					att.setName(Attributes.LABEL_NAME);
					label = att;
				}
			}
		}
		// include file name ?
		if (includeFileName) {

			Attribute att = AttributeFactory.createAttribute("FileName",
					Ontology.STRING);
			attributes.add(att);
			specialAttribs.add(att);
		}
		if (includePath) {
			Attribute att = AttributeFactory.createAttribute("FilePath",
					Ontology.STRING);
			attributes.add(att);
			specialAttribs.add(att);
		}
		if (includePathFileName) {
			Attribute att = AttributeFactory.createAttribute(
					"FilePathPlusName", Ontology.STRING);
			attributes.add(att);
			specialAttribs.add(att);
		}
		MemoryExampleTable table = new MemoryExampleTable(attributes);

		ExampleSet es = table.createExampleSet();

		// Mark fitness as label
		if (label != null) {
			// always the same name
			// label.setName(Attributes.LABEL_NAME);
			es.getAttributes().setLabel(label);
		}

		// mark attributes to be ignored
		for (Attribute attribute : specialAttribs) {
			es.getAttributes().setSpecialAttribute(attribute,
					attribute.getName());
		}
		return es;
	}
}
