/*
 *  Brno University of Technology
 *
 *  Copyright (C) 2009-2010 by Brno University of Technology and the contributors
 *
 *  Complete list of developers available at our web site:
 *
 *       http://spl.utko.feec.vutbr.cz
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.
 *
 *  You should have received a copy of the GNU Affero General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/.
 */

package cz.vutbr.feec.imageprocessing.op.featureExtraction.interestPoints;

import ij.process.ImageProcessor;
import Jama.Matrix;

import com.rapidminer.operator.Operator;
import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.operator.OperatorException;
import com.rapidminer.operator.ports.InputPort;
import com.rapidminer.operator.ports.OutputPort;
import com.rapidminer.operator.ports.metadata.MetaData;
import com.rapidminer.operator.ports.metadata.SimplePrecondition;

import cz.vutbr.feec.imageprocessing.imagej.imageplus.BaseImagePlusIOObject;
import cz.vutbr.feec.imageprocessing.imagej.imageplus.ImagePlusGSIOObject;
import cz.vutbr.feec.imageprocessing.imagej.imageplus.IntegralImageIOObject;
import cz.vutbr.feec.imageprocessing.imagej.imageplus.PointsIOObject;

public class IntegralImageOperator extends Operator {
	private InputPort imageInput = getInputPorts().createPort("grayscale image plus");
	private OutputPort integralImageOutput = getOutputPorts().createPort("integral image");
	
	public IntegralImageOperator(OperatorDescription description) {
		super(description);
		imageInput.addPrecondition(new SimplePrecondition(imageInput, new MetaData(
				ImagePlusGSIOObject.class)));
		getTransformer().addGenerationRule(integralImageOutput, IntegralImageIOObject.class);
	}

	public void doWork() throws OperatorException {
		ImagePlusGSIOObject imageIO = imageInput.getData();
		IntegralImageIOObject integralImageIO = new IntegralImageIOObject();
		integralImageIO.setIntegralImage(getIntegralImage(imageIO));
		integralImageOutput.deliver(integralImageIO);				
	}
	
	private Matrix getIntegralImage (ImagePlusGSIOObject imageIO) {
		Matrix integralImage = new Matrix (imageIO.getImage().getHeight(),imageIO.getImage().getWidth());
		
		double sum;
		int width = imageIO.getImage().getWidth();
		int height = imageIO.getImage().getHeight();
		ImageProcessor processor = imageIO.getImage().getProcessor();
		
		// create the integral image			
		for (int y = 0; y < height; y++)
		{
			// reset this column sum
			sum = 0;
			for (int x = 0; x < width; x++)
			{
				sum += processor.getPixel(y, x);
				if (y == 0)
					integralImage.set(x, y, sum);
				else
					integralImage.set(x, y, integralImage.get(x, y-1) + sum);
			}
		}		
		return integralImage;
	}
}
