/*
 *  Brno University of Technology
 *
 *  Copyright (C) 2009-2010 by Brno University of Technology and the contributors
 *
 *  Complete list of developers available at our web site:
 *
 *       http://spl.utko.feec.vutbr.cz
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.
 *
 *  You should have received a copy of the GNU Affero General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/.
 */
package cz.vutbr.feec.imageprocessing.op.featureExtraction.interestPoints;


import java.awt.image.BufferedImage;
import java.util.List;


import com.rapidminer.operator.Operator;
import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.operator.OperatorException;
import com.rapidminer.operator.ports.InputPort;
import com.rapidminer.operator.ports.OutputPort;
import com.rapidminer.operator.ports.metadata.MetaData;
import com.rapidminer.operator.ports.metadata.SimplePrecondition;
import com.rapidminer.parameter.ParameterType;
import com.rapidminer.parameter.ParameterTypeBoolean;
import com.rapidminer.parameter.ParameterTypeDouble;
import com.rapidminer.parameter.ParameterTypeInt;

import cz.vutbr.feec.imageprocessing.imagej.imageplus.ImagePlusGSIOObject;
import cz.vutbr.feec.imageprocessing.imagej.imageplus.SURFInterestPointsIOObject;

public class SURFInterestPoints extends Operator {
	
	private final String PARAM_HESSIAN_BALANCE_VALUE = "hessian_balance_value";
	
	private final String PARAM_HESSIAN_THRESHOLD = "hessian_threshold";
	
	private final String PARAM_OCTAVES = "number_of_octaves";	
	
	private final String PARAM_UPRIGHT = "upright_version";
	
	private InputPort imageInput = getInputPorts().createPort("grayscale image plus");
	private OutputPort pointsOutput = getOutputPorts().createPort("surf interest points");

	public SURFInterestPoints(OperatorDescription description) {
		super(description);
		imageInput.addPrecondition(new SimplePrecondition(imageInput,
				new MetaData(ImagePlusGSIOObject.class)));
		getTransformer().addGenerationRule(pointsOutput, SURFInterestPointsIOObject.class);
		// imageInput.addPrecondition(new SimplePrecondition("image plus",
		// ImagePlusIOObject.class))
	}

	@Override
	public void doWork() throws OperatorException {
		ImagePlusGSIOObject imageIO = imageInput.getData();
		BufferedImage bufim = imageIO.getImage().getBufferedImage();
		Surf surf = new Surf(bufim);
		boolean upright = getParameterAsBoolean(PARAM_UPRIGHT);
		List<SURFInterestPoint> points = null;
		if(upright)
			points= surf.getUprightInterestPoints();
		else
			points = surf.getFreeOrientedInterestPoints();
		
		SURFInterestPointsIOObject ioObject = new SURFInterestPointsIOObject(bufim.getWidth(), bufim.getHeight());
		ioObject.addAll(points);		
		pointsOutput.deliver(ioObject);
	}

	@Override
	public List<ParameterType> getParameterTypes() {
		List<ParameterType> types = super.getParameterTypes();
		types.add(new ParameterTypeBoolean(PARAM_UPRIGHT, "Use the upright (faster) version", true));
		types.add(new ParameterTypeDouble(PARAM_HESSIAN_BALANCE_VALUE,
				"Hessian balance value", 0, 999999999, 0.81F));
		types.add(new ParameterTypeDouble(PARAM_HESSIAN_THRESHOLD,
				"Hessian threshold", 0, 999999999, 0.0004F));
		types.add(new ParameterTypeInt(PARAM_OCTAVES,
				"Number of octaves", 0, 10, 5));
		return types;
	}
}
